class TestWordIndex {
  private static int line;
  public static void main(String[] args) {
    WordIndex[] index = new WordIndex[] { 
      new WordIndexList(),
      new WordIndexTree(),
      new WordIndexHashMap(WordIndexHashMap.CollisionResolution.LINEAR_PROBING),
      new WordIndexHashMap(WordIndexHashMap.CollisionResolution.QUADRATIC_PROBING),
      new WordIndexHashMap(WordIndexHashMap.CollisionResolution.SEPARATE_CHAINING),
    };
    for(WordIndex i: index) {
      Out.println("\n******* " + i.getClass().getName() + " *******");
      testAlice(i);
      //testHamlet(i);
    }
  }
  public static void testHamlet(WordIndex index) {
    // TO BE
    // OR NOT TO BE
    index.insert("TO", 1);
    index.insert("BE", 1);
    index.insert("OR", 2);
    index.insert("NOT", 2);
    index.insert("TO", 2);
    index.insert("BE", 2);
    Out.println("Total Words=" + index.getTotalWordCount());
    Out.println("Different Words=" + index.getDifferentWordCount());
    for(WordData w: index) {
      Out.print(String.format("\n%s (%d times):", 
        w.getWord(), w.getFrequency()));
      for(int paragraph: w.getParagraphs()) {
        Out.print(" " + paragraph);
      }
    }
    Out.println(index);
  }
  public static void testAlice(WordIndex index) {
    In.open("alice.txt");
    line = 1;
    String w = readWord();
    int cnt = 0;
    while(In.done()) {
      w = w.toLowerCase();
      index.insert(w, line);
      w = readWord();
    }
    Out.println("Total Words=" + index.getTotalWordCount());
    Out.println("Different Words=" + index.getDifferentWordCount());
    Out.println("Mean Frequency=" + index.getMeanWordFrequency());
    Out.println("Frequency (\"alice\")=" + index.getFrequency("alice"));
    Out.println("Frequency (\"altogether\")=" + index.getFrequency("altogether"));
    Out.println("Frequency (\"always\")=" + index.getFrequency("always"));
    Out.println("\nWord with frequency >= 500");
    for(String word: index.getWordsWithMinFrequency(500)) {
      Out.println(String.format("%s (%d)", word, index.getFrequency(word)));
    }
    Out.println("\nWords starting with Al");
    for(String word: index.getWordsStartingWith("al")) {
      Out.println(String.format("%s (%d)", word, index.getFrequency(word)));
    }
    In.close();
  }
  private static String readWord() {
    StringBuilder b = new StringBuilder();
    char ch;
    // read over non-words
    do {
      ch = In.read();
      if(ch == '\n') line++;
    } while(In.done() && !Character.isLetter(ch));
    // read word until whitespace
    while(In.done() && Character.isLetter(ch)) {
      b.append(ch);
      ch = In.read();
      if(ch == '\'') {
        do {
          ch = In.read();
          if(ch == '\n') line++;
        } while(!Character.isWhitespace(ch));
      }
    }
    return b.toString();
  }
  public static void printWord(WordData word) {
    Out.print(String.format("%s (%dx)", word.getWord(), word.getFrequency()));
    printLines(word);
    Out.println();
  }
  private static void printLines(WordData word) {
    boolean first = true;
    for(int i: word.getParagraphs()) {
      if(first) {
        first = false;
        Out.print(" ");
      } else {
        Out.print(", ");
      }
      Out.print(i);
    }
  }
}